<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayer\Impressions;

use Flat101\EnhancedEcommerce\Api\Data\DataLayerInterface;
use Flat101\EnhancedEcommerce\Model\Handler\CatalogImpressionHandler;
use Flat101\EnhancedEcommerce\Model\Session;
use Magento\Framework\Event\Manager;
use Magento\Catalog\Model\Layer\Resolver;
use Magento\Framework\App\Request\Http;

class CatalogImpressionView implements DataLayerInterface
{
    public const CODE = 'catalog-impression-view';

    /** @var Session */
    private $session;

    /** @var Manager */
    private $eventManager;

    /** @var Resolver */
    private $catalogLayer;

    /** @var Http */
    private $request;

    /** @var CatalogImpressionHandler */
    private $impressionHandler;

    public function __construct(
        Session $session,
        Manager $eventManager,
        Resolver $layerResolver,
        Http $request,
        CatalogImpressionHandler $impressionHandler
    ) {
        $this->session = $session;
        $this->eventManager = $eventManager;
        $this->catalogLayer = $layerResolver->get();
        $this->request = $request;
        $this->impressionHandler = $impressionHandler;
    }

    public function getCode(): string
    {
        return static::CODE;
    }

    /**
     * {@inheritdoc}
     */
    public function getLayer(): array
    {
        // Amasty Brands: Add impressions info
        $handle = $this->request->getFullActionName();
        if ($handle === 'ambrand_index_index') {
            $this->impressionHandler->handle($this->catalogLayer->getProductCollection());
        }

        $products = $this->session->getImpressionCatalogProducts(true,false);

        $impression = 0 === \count($products) ? [] : $products;

        $this->eventManager->dispatch('flat101_enhancedecommerce_catalog_impression_view', [
            'dataLayer' => $impression,
        ]);

        return $impression;
    }
}
